﻿<#
.SYNOPSIS
Script for setting up SQL failover cluster

.DESCRIPTION
This script will configure failover clustering to support a SQL failover cluster.


#>
param
(
	#root cluster object name
	[Parameter(Mandatory=$true)]
	[string]$clustername,
	
	#root cluster object ip address
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match [ipaddress] $_})]
	[string]$clusterIP = "0.0.0.0",
	
	#quorum disk number
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match "\d+"})]
	[int]$quorumdisknum = 1,
	
	#quorum disk drive letter
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match "[A-Za-z]"})]
	[string]$quorumdiskletter = "",
	
	#data disk number
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match "\d+"})]
	[int]$datadisknum = 2,
	
	#data disk drive letter
	[Parameter(Mandatory=$false)]
    [ValidateScript({ $_ -match "[A-Za-z]"})]
	[string]$datadiskletter = "",
	
	#first node in cluster switch
	[switch]$FirstNode,
	
	#debug mode
	[switch]$VerboseMode
);

function Log($out) { 
    $out = [System.DateTime]::Now.ToString("yyyy.MM.dd hh:mm:ss") + " ---- " + $out; 
    Write-Output "$out"; 
}

function Log-Verbose($out) {
	if ($VerboseMode) {
		Log $out;
	}
}

function Create-Disk($disk, $letter) {
	if ($letter -eq "") {
		$letter = $script:letters[$disk.Number];
	}
	Log-Verbose "Preparing Disk: Num: $($disk.Number), Size (MB): $("{0:N0}" -f $($disk.Size / 1MB))"
	Log-Verbose "Bringing drive online..."; 
	Set-Disk -Number $disk.Number -IsOffline $False;
	Log-Verbose "Initializing partition table as MBR...";
	Initialize-Disk -Number $disk.Number -PartitionStyle MBR;
	Log-Verbose "Creating single partition...";
	New-Partition $disk.Number -UseMaximumSize | Out-Null;
	Log-Verbose "Formatting $($disk.Name) (Drive $($disk.Number)) and assigning drive letter $letter..."; 
    Set-Partition $disk.Number -PartitionNumber 1 -NewDriveLetter $letter | Out-Null; 
    Format-Volume -DriveLetter $letter -FileSystem NTFS -Confirm:$False | Out-Null; 
}

Log-Verbose "Parameters";
Log-Verbose "Cluster Name:`t`t$clustername";
Log-Verbose "Cluster IP:`t`t$clusterIP";
Log-Verbose "Quorum Disk Number:`t$quorumdisknum";
Log-Verbose "Quorum Disk Letter:`t$quorumdiskletter";
Log-Verbose "Data Disk Number:`t$datadisknum";
Log-Verbose "Data Disk Letter:`t$datadiskletter";
Log-Verbose "First Node Flag:`t$FirstNode";

$script:letters = 67..90 | %{[char]$_} | ?{!(Get-PSDrive -Name $_ -ErrorAction SilentlyContinue)}; 

#Cluster Configuration
if ($FirstNode) {
	if($clusterIP -ne "0.0.0.0") { 
		Log "Creating cluster $clustername on $env:COMPUTERNAME with IP $clusterIP"; 
		New-Cluster -Name $clustername -Node localhost -StaticAddress $clusterIP | Out-Null; 
	} else { 
		Log "Creating cluster $clustername on $env:COMPUTERNAME using DHCP"; 
		New-Cluster -Name $clustername -Node localhost | Out-Null; 
	}
	
	Log "Getting available disks..."; 
	$disks = Get-Disk | ? FriendlyName -eq "Microsoft Virtual Disk";
	$quorumdisk = $disks | ? Number -eq $quorumdisknum;
	$datadisk = $disks | ? Number -eq $datadisknum;
	
	if (-not $quorumdisk -or -not $datadisk) {
		Write-Error "Error locating quorum or datadisk";
		$($disks | ft);
		exit -1;
	}
	
	Log "Found quorum disk...";
	Create-Disk $quorumdisk $quorumdiskletter;
	Log "Found data disk...";
	Create-Disk $datadisk $datadiskletter;
	
	Log "Adding disks to cluster..."; 
	$clusterdisks = @(Get-ClusterAvailableDisk);
	$quorumdiskname = ($clusterdisks | ? Number -eq $quorumdisknum).Name;
	$clusterdisks | Add-ClusterDisk; 
	for($i=0;$i -lt 12;$i++) {
		if((Get-ClusterResource $quorumdiskname).State -eq "Online") {
			break;
		}
		Log-Verbose "Waiting for quorum disk to come online...";
		Start-Sleep -s 5
	}
	if($i -eq 12) {
		Write-Error "Timeout waiting for quorum disk to come online";
		$(Get-ClusterResource $quorumdiskname | ft);
		exit -1;
	}
	Log "Assigning witness disk..."; 
	Log-Verbose "Quorum Disk Cluster Resource: $quorumdiskname";
	Set-ClusterQuorum -NodeAndDiskMajority "$quorumdiskname"; 
}
else {
	for($i=0;$i -lt 6;$i++) {
		if ($([System.Net.Dns]::GetHostAddresses($clustername))) {
			break;
		}
		Log-Verbose "Waiting for DNS..."; 
		Start-Sleep -s 20
	}

	Log "Adding localhost to $clustername"; 
	Get-Cluster -Name $clustername | Add-ClusterNode -Name localhost; 
}

Log "Validating cluster..."; 
$results = Test-Cluster -WarningAction SilentlyContinue; 
if(Select-String '<div class=3D"error">' $results) { 
	Log "Cluster Validation errors."; 
	Write-Error "Validation failure. See '$results' for details."; 
	exit -1; 
} 

Log "Validation complete." 
